@@ -23,8 +23,9 @@
 """
 A tile source that serves OpenStreetMap tiles from server(s).
 """
+from __future__ import  annotations
 import math
-from typing import Tuple
+from typing import Tuple, List
 from GridCal.Gui.Diagrams.MapWidget.Tiles.tiles import Tiles
 
 
@@ -33,7 +34,7 @@ class CartoDbTiles(Tiles):
 
     def __init__(self, tiles_dir='open_street_map_tiles',
                  http_proxy=None,
-                 tile_servers=None,
+                 tile_servers: List[str] | None = None,
                  name: str = 'Carto positron'):
         """
         Override the base class for these tiles.
@@ -52,22 +53,38 @@ class CartoDbTiles(Tiles):
                          tile_width=256,
                          tile_height=256,
                          tiles_dir=tiles_dir,
-                         servers=[
-                             "https://basemaps.cartocdn.com/dark_all/"
-                         ] if tile_servers is None else tile_servers,
+                         servers=["https://basemaps.cartocdn.com/dark_all/"] if tile_servers is None else tile_servers,
                          url_path='/{Z}/{X}/{Y}.png',
                          max_server_requests=2,
                          max_lru=10000,
                          http_proxy=http_proxy,
                          attribution="© CARTO, © OpenStreetMap contributors")
-        # TODO: implement map wrap-around
-        #        self.wrap_x = True
-        #        self.wrap_y = False
 
         # get tile information into instance
         self.level = min(self.levels)
         self.num_tiles_x, self.num_tiles_y, self.ppd_x, self.ppd_y = self.GetInfo(self.level)
 
+    def copy(self) -> "CartoDbTiles":
+        """
+        Copy of this object
+        :return:
+        """
+        # cpy = super().copy()
+        # cpy.__class__ == CartoDbTiles
+        # cpy: CartoDbTiles = cpy
+        # cpy.level = self.level
+        # cpy.num_tiles_x = self.num_tiles_x
+        # cpy.num_tiles_y = self.num_tiles_y
+        # cpy.ppd_x = self.ppd_x
+        # cpy.ppd_y = self.ppd_y
+
+        cpy = CartoDbTiles(tiles_dir=self.tiles_dir,
+                           http_proxy=self.http_proxy,
+                           tile_servers=self.servers.copy(),
+                           name=self.tile_set_name)
+
+        return cpy
+
     def Geo2Tile(self, longitude: float, latitude: float) -> Tuple[float, float]:
         """
         Convert geo to tile fractional coordinates for level in use.
