@@ -55,12 +55,6 @@ class Tiles(BaseTiles):
     A tile object to source server tiles for the widget.
     """
 
-    # allowed file types and associated values
-    AllowedFileTypes = {'png': 'PNG', 'jpg': 'JPG'}
-
-    # the number of seconds in a day
-    SecondsInADay = 60 * 60 * 24
-
     def __init__(self,
                  tile_set_name: str,
                  tile_set_short_name: str,
@@ -92,6 +86,15 @@ class Tiles(BaseTiles):
         :param http_proxy: proxy to use if required
         :param re_fetch_days: fetch new server tile if older than this in days (0 means don't ever update tiles)
         """
+        # perform the base class initialization
+        super().__init__(levels, tile_width, tile_height, tiles_dir, max_lru)
+
+        # allowed file types and associated values
+        self.AllowedFileTypes = {'png': 'PNG', 'jpg': 'JPG'}
+
+        # the number of seconds in a day
+        self.SecondsInADay = 60 * 60 * 24
+
         self.tile_set_name = tile_set_name
         self.tile_set_short_name = tile_set_short_name
         self.tile_set_version = tile_set_version
@@ -105,9 +108,6 @@ class Tiles(BaseTiles):
             if not os.path.isdir(level_dir):
                 os.makedirs(level_dir)
 
-        # perform the base class initialization
-        super().__init__(levels, tile_width, tile_height, tiles_dir, max_lru)
-
         # save params not saved in super()
         self.servers = servers
         self.url_path = url_path
@@ -221,6 +221,26 @@ class Tiles(BaseTiles):
                 self.workers.append(worker)
                 worker.start()
 
+    # def copy(self) -> "Tiles":
+    #     """
+    #     Perform a copy of the Tiles object.
+    #     :return:
+    #     """
+    #     return Tiles(tile_set_name=self.tile_set_name,
+    #              tile_set_short_name=self.tile_set_short_name,
+    #              tile_set_version=self.tile_set_version,
+    #              levels=self.levels.copy(),
+    #              tile_width=self.tile_width,
+    #              tile_height=self.tile_height,
+    #              tiles_dir=self.tiles_dir,
+    #              max_lru=self.max_lru,
+    #              servers=self.servers.copy(),
+    #              url_path=self.url_path,
+    #              max_server_requests=self.max_requests,
+    #              http_proxy=self.http_proxy,
+    #              re_fetch_days=self.refresh_tiles_after_days,
+    #              attribution=self.attribution_string)
+
     def UseLevel(self, level):
         """
         Prepare to serve tiles from the required level.
