@@ -55,7 +55,7 @@ from PySide6.QtGui import (QPainter, QColor, QPixmap, QCursor,
                            QResizeEvent, QEnterEvent, QPaintEvent, QDragEnterEvent, QDragMoveEvent, QDropEvent)
 from PySide6.QtWidgets import (QSizePolicy, QWidget, QGraphicsScene, QGraphicsView, QStackedLayout,
                                QGraphicsSceneMouseEvent, QGraphicsItem, QLabel, QGraphicsProxyWidget)
-# from PySide6.QtOpenGLWidgets import QOpenGLWidget
+
 from GridCal.Gui.Diagrams.MapWidget.Tiles.tiles import Tiles
 
 if TYPE_CHECKING:
@@ -84,7 +84,8 @@ class MapDiagramScene(QGraphicsScene):
     """
 
     def __init__(self, parent: "MapWidget" = None) -> None:
-        super(MapDiagramScene, self).__init__(parent)
+        super().__init__(parent)
+
         self.setItemIndexMethod(QGraphicsScene.ItemIndexMethod.BspTreeIndex)  # For efficient item indexing
 
     def mousePressEvent(self, event: QGraphicsSceneMouseEvent) -> None:
@@ -127,7 +128,7 @@ class MapView(QGraphicsView):
         :param scene:
         :param map_widget:
         """
-        QGraphicsView.__init__(self, scene)
+        super().__init__(scene)
 
         self._scene: QGraphicsScene = scene
 
@@ -452,7 +453,7 @@ class MapWidget(QWidget):
         :param position_callback: position change callback function
         """
 
-        QWidget.__init__(self, parent)  # inherit all parent object setup
+        super().__init__(parent)  # inherit all parent object setup
 
         # this is where you draw
         self.diagram_scene = MapDiagramScene(self)
@@ -485,7 +486,7 @@ class MapWidget(QWidget):
         # Internal vars
         # -------------------------------------------------------------------------
         # remember the tile source object
-        self._tile_src: Tiles = tile_src
+        self._tile_src: Tiles = tile_src.copy()
         self._tile_src.setCallback(self.on_tile_available)
         self.view.set_notice(val=self._tile_src.attribution_string)
 
@@ -579,7 +580,7 @@ class MapWidget(QWidget):
         level, longitude, latitude = self.get_level_and_position()
 
         if tile_src.tile_set_name != self._tile_src.tile_set_name:  # avoid changing tile sets to themselves
-            self._tile_src: Tiles = tile_src
+            self._tile_src: Tiles = tile_src.copy()
             self._tile_src.setCallback(self.on_tile_available)
             self.view.set_notice(val=self._tile_src.attribution_string)
 
@@ -721,9 +722,9 @@ class MapWidget(QWidget):
         Could be end of a drag or point or box selection.  If it's the end of
         a drag we don't do a lot.  If a selection we process that.
         """
-
-        x = event.x()
-        y = event.y()
+        pos = event.position()
+        x = pos.x()
+        y = pos.y()
 
         # cursor back to normal in case it was a box select
         self.setCursor(self.default_cursor)
@@ -782,8 +783,9 @@ class MapWidget(QWidget):
         If we are off the map, ensure self.mouse_x, etc, are None.
         """
         if not self.block_movement:
-            x = event.x()
-            y = event.y()
+            pos = event.position()
+            x = pos.x()
+            y = pos.y()
 
             mouse_geo = self.view_to_geo(x, y)
 
@@ -1756,7 +1758,7 @@ class MapWidget(QWidget):
                 result = (r, g, b, a)
         elif isinstance(colour, QColor):
             # if it's a QColor, get float RGBA values, convert to ints
-            result = [int(v * 255) for v in colour.getRgbF()]
+            result = [int(v * 255) for v in tuple(colour.getRgbF())]
         else:
 
             # we assume a list or tuple
