@@ -134,10 +134,18 @@ class GeneratorGraphicItem(InjectionTemplateGraphicItem):
 
         menu.addSeparator()
 
-        add_menu_entry(menu=menu,
-                       text="Delete",
-                       icon_path=":/Icons/icons/delete3.svg",
-                       function_ptr=self.remove)
+        # add_menu_entry(menu=menu,
+        #                text="Delete",
+        #                icon_path=":/Icons/icons/delete3.svg",
+        #                function_ptr=self.remove)
+
+        add_menu_entry(menu, text='Remove from schematic and DB',
+                       icon_path=":/Icons/icons/delete_db.svg",
+                       function_ptr=lambda: self.remove_from_widget_and_db(ask=True, delete_from_db=True))
+
+        add_menu_entry(menu, text='Remove from schematic',
+                       icon_path=":/Icons/icons/delete_schematic.svg",
+                       function_ptr=lambda: self.remove_from_widget_and_db(ask=True, delete_from_db=False))
 
         add_menu_entry(menu=menu,
                        text="Convert to battery",
@@ -272,9 +280,7 @@ class GeneratorGraphicItem(InjectionTemplateGraphicItem):
 
         if self.editor.circuit.has_time_series:
 
-            time_array = self.editor.circuit.time_profile
-
-            dlg = SolarPvWizard(time_array=time_array,
+            dlg = SolarPvWizard(time_array=self.editor.circuit.time_profile.strftime("%Y-%m-%d %H:%M").tolist(),
                                 peak_power=self.api_object.Pmax,
                                 latitude=self.api_object.bus.latitude,
                                 longitude=self.api_object.bus.longitude,
@@ -299,9 +305,7 @@ class GeneratorGraphicItem(InjectionTemplateGraphicItem):
 
         if self.editor.circuit.has_time_series:
 
-            time_array = self.editor.circuit.time_profile
-
-            dlg = WindFarmWizard(time_array=time_array,
+            dlg = WindFarmWizard(time_array=self.editor.circuit.time_profile.strftime("%Y-%m-%d %H:%M").tolist(),
                                  peak_power=self.api_object.Pmax,
                                  latitude=self.api_object.bus.latitude,
                                  longitude=self.api_object.bus.longitude,
