@@ -5,6 +5,7 @@
 from __future__ import annotations
 import numpy as np
 from typing import Union, TYPE_CHECKING, List, Dict
+import webbrowser
 from PySide6 import QtWidgets
 from PySide6.QtCore import Qt, QRectF, QRect, QPointF
 from PySide6.QtGui import QPen, QCursor, QIcon, QPixmap, QBrush, QColor
@@ -456,7 +457,7 @@ class BusGraphicItem(GenericDiagramWidget, QtWidgets.QGraphicsRectItem):
 
         add_menu_entry(menu, text='Delete all the connections',
                        icon_path=":/Icons/icons/delete_conn.svg",
-                       function_ptr=self.delete_all_connections)
+                       function_ptr=lambda: self.delete_all_connections(ask=True, delete_from_db=True))
 
         add_menu_entry(menu, text='Remove from schematic and DB',
                        icon_path=":/Icons/icons/delete_db.svg",
@@ -474,6 +475,11 @@ class BusGraphicItem(GenericDiagramWidget, QtWidgets.QGraphicsRectItem):
                        icon_path=":/Icons/icons/grid_icon.svg",
                        function_ptr=self.new_vecinity_diagram_from_here)
 
+        add_menu_entry(menu=menu,
+                       text="Open in street view",
+                       icon_path=":/Icons/icons/map.svg",
+                       function_ptr=self.open_street_view)
+
         menu.addSection("Add")
 
         # Actions under the "Add" section
@@ -519,11 +525,18 @@ class BusGraphicItem(GenericDiagramWidget, QtWidgets.QGraphicsRectItem):
         """
         self.editor.set_active_status_to_profile(self.api_object)
 
-    def delete_all_connections(self) -> None:
+    def delete_all_connections(self, ask: bool, delete_from_db: bool) -> None:
         """
         Delete all bus connections
         """
-        self._terminal.remove_all_connections()
+        if ask:
+            ok = yes_no_question('Are you sure that you want to remove this bus',
+                                 'Remove bus from schematic and DB' if delete_from_db else "Remove bus from schematic")
+        else:
+            ok = True
+
+        if ok:
+            self._terminal.remove_all_connections(delete_from_db=delete_from_db)
 
     def remove_from_widget_and_db(self, ask: bool = True, delete_from_db: bool = True) -> None:
         """
@@ -532,16 +545,11 @@ class BusGraphicItem(GenericDiagramWidget, QtWidgets.QGraphicsRectItem):
         """
         if ask:
             ok = yes_no_question('Are you sure that you want to remove this bus',
-                                 'Remove bus from schamatic and DB' if delete_from_db else "Remove bus from schamatic")
+                                 'Remove bus from schematic and DB' if delete_from_db else "Remove bus from schematic")
         else:
             ok = True
 
         if ok:
-            # self.delete_all_connections()
-            #
-            # for g in self.shunt_children:
-            #     self.editor.remove_from_scene(g.nexus)
-
             self.editor.remove_element(device=self.api_object,
                                        graphic_object=self,
                                        delete_from_db=delete_from_db)
@@ -900,6 +908,18 @@ class BusGraphicItem(GenericDiagramWidget, QtWidgets.QGraphicsRectItem):
 
         self.setToolTip(msg)
 
+    def open_street_view(self):
+        """
+        Call open street maps
+        :return:
+        """
+        # https://maps.google.com/?q=<lat>,<lng>
+        if self.api_object is not None:
+            url = f"https://www.google.com/maps/?q={self.api_object.latitude},{self.api_object.longitude}"
+            webbrowser.open(url)
+        else:
+            warning_msg(f"No API object available :(")
+
     def __str__(self):
 
         if self.api_object is None:
