@@ -11,6 +11,7 @@ from PySide6.QtGui import QPen, QCursor, QIcon, QPixmap
 from PySide6.QtWidgets import QMenu, QGraphicsSceneMouseEvent
 
 from GridCal.Gui.messages import yes_no_question
+from GridCal.Gui.gui_functions import add_menu_entry
 from GridCal.Gui.Diagrams.generic_graphics import (GenericDiagramWidget, ACTIVE, FONT_SCALE)
 from GridCal.Gui.Diagrams.SchematicWidget.terminal_item import RoundTerminalItem
 from GridCal.Gui.Diagrams.SchematicWidget.Injections.load_graphics import LoadGraphicItem, Load
@@ -94,22 +95,22 @@ class CnGraphicItem(GenericDiagramWidget, QtWidgets.QGraphicsRectItem):
 
         # connection terminals the block
         self._terminal = RoundTerminalItem('s', parent=self, editor=self.editor, h=20, w=20)  # , h=self.h))
-        self._terminal.setPen(QPen(Qt.transparent, self.pen_width, self.style, Qt.RoundCap, Qt.RoundJoin))
+        self._terminal.setPen(QPen(Qt.GlobalColor.transparent, self.pen_width, self.style, Qt.PenCapStyle.RoundCap, Qt.PenJoinStyle.RoundJoin))
         self._terminal.setPos(QPoint(15, 15))
         self._terminal_mid_point = QPoint(20, 20)  # (15, 15) + (20, 20) / 2
 
-        self.setPen(QPen(Qt.transparent, self.pen_width, self.style))
-        self.setBrush(Qt.transparent)
+        self.setPen(QPen(Qt.GlobalColor.transparent, self.pen_width, self.style))
+        self.setBrush(Qt.GlobalColor.transparent)
         # self.setBrush(QBrush(QColor(255, 0, 0)))
         self.setFlags(self.GraphicsItemFlag.ItemIsSelectable | self.GraphicsItemFlag.ItemIsMovable)
-        self.setCursor(QCursor(Qt.PointingHandCursor))
+        self.setCursor(QCursor(Qt.CursorShape.PointingHandCursor))
 
         self.set_position(x, y)
         self.setRect(0.0, 0.0, self.w, self.h)
 
     def get_nexus_point(self) -> QPointF:
         """
-        Get the connection point for the chldren nexus line
+        Get the connection point for the children nexus line
         :return: QPointF
         """
         return QPointF(self.x() + self._terminal_mid_point.x(),
@@ -143,17 +144,17 @@ class CnGraphicItem(GenericDiagramWidget, QtWidgets.QGraphicsRectItem):
                                            draw_labels=self.draw_labels,
                                            graphic_object=self)
 
-    def set_position(self, x: int, y: int) -> None:
+    def set_position(self, x: float, y: float) -> None:
         """
         Set the bus x, y position
         :param x: x in pixels
         :param y: y in pixels
         """
         if np.isnan(x):
-            x = 0
+            x = 0.0
         if np.isnan(y):
-            y = 0
-        self.setPos(QPoint(int(x), int(y)))
+            y = 0.0
+        self.setPos(QPointF(x, y))
 
     def merge(self, other_bus_graphic: "CnGraphicItem") -> None:
         """
@@ -263,11 +264,9 @@ class CnGraphicItem(GenericDiagramWidget, QtWidgets.QGraphicsRectItem):
         arr.setIcon(arr_icon)
         arr.triggered.connect(self.arrange_children)
 
-        ra3 = menu.addAction('Delete all the connections')
-        del2_icon = QIcon()
-        del2_icon.addPixmap(QPixmap(":/Icons/icons/delete_conn.svg"))
-        ra3.setIcon(del2_icon)
-        ra3.triggered.connect(self.delete_all_connections)
+        add_menu_entry(menu, text='Delete all the connections',
+                       icon_path=":/Icons/icons/delete_conn.svg",
+                       function_ptr=lambda: self.delete_all_connections(ask=True, delete_from_db=True))
 
         da = menu.addAction('Delete')
         del_icon = QIcon()
@@ -327,11 +326,18 @@ class CnGraphicItem(GenericDiagramWidget, QtWidgets.QGraphicsRectItem):
 
         menu.exec_(event.screenPos())
 
-    def delete_all_connections(self) -> None:
+    def delete_all_connections(self, ask: bool, delete_from_db: bool) -> None:
         """
         Delete all bus connections
         """
-        self._terminal.remove_all_connections()
+        if ask:
+            ok = yes_no_question('Are you sure that you want to remove this bus',
+                                 'Remove bus from schematic and DB' if delete_from_db else "Remove bus from schematic")
+        else:
+            ok = True
+
+        if ok:
+            self._terminal.remove_all_connections(delete_from_db=delete_from_db)
 
     def remove(self, ask: bool = True) -> None:
         """
@@ -345,7 +351,7 @@ class CnGraphicItem(GenericDiagramWidget, QtWidgets.QGraphicsRectItem):
             ok = True
 
         if ok:
-            self.delete_all_connections()
+            self.delete_all_connections(ask=False, delete_from_db=True)
 
             for g in self.shunt_children:
                 self.editor.remove_from_scene(g.nexus)
