@@ -14,14 +14,12 @@ from PySide6.QtWidgets import QListView, QTableView, QVBoxLayout, QHBoxLayout, Q
 
 from GridCalEngine.Devices.types import ALL_DEV_TYPES
 from GridCalEngine.Devices.multi_circuit import MultiCircuit
-from GridCalEngine.Devices.Substation import Bus
 from GridCalEngine.Devices.Branches.line import Line
 from GridCalEngine.Devices.Branches.dc_line import DcLine
 from GridCalEngine.Devices.Branches.hvdc_line import HvdcLine
 from GridCalEngine.Devices.Branches.transformer import Transformer2W
 from GridCalEngine.Devices.Branches.vsc import VSC
 from GridCalEngine.Devices.Branches.upfc import UPFC
-from GridCalEngine.Devices.types import BRANCH_TYPES
 from GridCalEngine.Simulations import (PowerFlowTimeSeriesResults, LinearAnalysisTimeSeriesResults,
                                        ContingencyAnalysisTimeSeriesResults, OptimalPowerFlowTimeSeriesResults,
                                        StochasticPowerFlowResults)
@@ -29,7 +27,7 @@ from GridCalEngine.basic_structures import Vec, CxVec, IntVec
 from GridCalEngine.Devices.Diagrams.schematic_diagram import SchematicDiagram
 from GridCalEngine.Devices.Diagrams.map_diagram import MapDiagram
 from GridCalEngine.Simulations.types import DRIVER_OBJECTS
-from GridCalEngine.basic_structures import Logger, Mat
+from GridCalEngine.basic_structures import Logger
 from GridCalEngine.enumerations import SimulationTypes, ResultTypes
 import GridCalEngine.Devices.Diagrams.palettes as palettes
 
@@ -38,9 +36,10 @@ from GridCal.Gui.messages import yes_no_question, info_msg
 from GridCal.Gui.object_model import ObjectsModel
 
 if TYPE_CHECKING:
-    from GridCal.Gui.Diagrams.MapWidget.grid_map_widget import MapLibraryModel, GridMapWidget
-    from GridCal.Gui.Diagrams.SchematicWidget.schematic_widget import SchematicLibraryModel, SchematicWidget
+    from GridCal.Gui.Diagrams.MapWidget.grid_map_widget import MapLibraryModel
+    from GridCal.Gui.Diagrams.SchematicWidget.schematic_widget import SchematicLibraryModel
     from GridCal.Gui.Main.SubClasses.Model.diagrams import DiagramsMain
+    from GridCal.Gui.Main.GridCalMain import GridCalMainGUI
 
 
 def change_font_size(obj, font_size: int):
@@ -140,19 +139,16 @@ class BaseDiagramWidget(QSplitter):
     """
 
     def __init__(self,
-                 gui: DiagramsMain,
+                 gui: GridCalMainGUI | DiagramsMain,
                  circuit: MultiCircuit,
                  diagram: Union[SchematicDiagram, MapDiagram],
                  library_model: Union[MapLibraryModel, SchematicLibraryModel],
-                 time_index: Union[None, int] = None,
-                 call_delete_db_element_func: Callable[
-                     [Union[GridMapWidget, SchematicWidget], ALL_DEV_TYPES], None] = None):
+                 time_index: Union[None, int] = None):
         """
         Constructor
         :param circuit:
         :param diagram:
         :param time_index:
-        :param call_delete_db_element_func:
         """
         QSplitter.__init__(self)
 
@@ -219,15 +215,12 @@ class BaseDiagramWidget(QSplitter):
         # logger
         self.logger: Logger = Logger()
 
-        # This function is meant to be a master delete function that is passed to each diagram
-        # so that when a diagram deletes an element, the element is deleted in all other diagrams
-        self.call_delete_db_element_func = call_delete_db_element_func
-
         self.results_dictionary: Dict[SimulationTypes, DRIVER_OBJECTS] = dict()
 
         # video pointer
         self._video: Union[None, cv2.VideoWriter] = None
 
+
     @property
     def name(self):
         """
