@@ -3,7 +3,7 @@
 # file, You can obtain one at https://mozilla.org/MPL/2.0/.
 # SPDX-License-Identifier: MPL-2.0
 import numpy as np
-from typing import Union, List, Set
+from typing import Union, List, Set, Tuple
 from PySide6 import QtGui, QtCore, QtWidgets
 from matplotlib import pyplot as plt
 from matplotlib.colors import LinearSegmentedColormap
@@ -26,10 +26,9 @@ from GridCal.Gui.messages import yes_no_question, error_msg, warning_msg, info_m
 from GridCal.Gui.Main.SubClasses.Model.diagrams import DiagramsMain
 from GridCal.Gui.TowerBuilder.LineBuilderDialogue import TowerBuilderGUI
 from GridCal.Gui.general_dialogues import LogsDialogue
-from GridCal.Gui.Diagrams.MapWidget.grid_map_widget import GridMapWidget
 from GridCal.Gui.SystemScaler.system_scaler import SystemScaler
-from GridCal.Gui.Diagrams.SchematicWidget.schematic_widget import (SchematicWidget,
-                                                                   make_diagram_from_buses)
+from GridCal.Gui.Diagrams.MapWidget.grid_map_widget import GridMapWidget, make_diagram_from_substations
+from GridCal.Gui.Diagrams.SchematicWidget.schematic_widget import SchematicWidget, make_diagram_from_buses
 
 
 class ObjectsTableMain(DiagramsMain):
@@ -296,7 +295,7 @@ class ObjectsTableMain(DiagramsMain):
         mdl = self.get_current_objects_model_view()
         if mdl is not None:
             mdl.copy_to_clipboard()
-            print('Copied!')
+            self.show_info_toast('Copied!')
         else:
             warning_msg('There is no data displayed, please display one', 'Copy profile to clipboard')
 
@@ -378,7 +377,7 @@ class ObjectsTableMain(DiagramsMain):
         else:
             return list()
 
-    def get_selected_table_buses(self) -> Set[dev.Bus]:
+    def get_selected_table_buses(self) -> Tuple[Set[dev.Bus], List[ALL_DEV_TYPES]]:
         """
         Get the list of selected buses, regardless of the object table type
         If the object has buses, this one takes them
@@ -386,6 +385,7 @@ class ObjectsTableMain(DiagramsMain):
         """
         model = self.ui.dataStructureTableView.model()
         buses = set()
+        selected_objects: List[ALL_DEV_TYPES] = list()
 
         if model is not None:
 
@@ -401,6 +401,7 @@ class ObjectsTableMain(DiagramsMain):
                     for idx in unique:
 
                         sel_obj: ALL_DEV_TYPES = model.objects[idx]
+                        selected_objects.append(sel_obj)
 
                         if isinstance(sel_obj, dev.Bus):
                             root_bus = sel_obj
@@ -438,13 +439,80 @@ class ObjectsTableMain(DiagramsMain):
                         elif isinstance(sel_obj, dev.Switch):
                             root_bus = sel_obj.bus_from
 
+                        elif isinstance(sel_obj, dev.VoltageLevel):
+                            root_bus = None
+                            sel = self.circuit.get_voltage_level_buses(vl=sel_obj)
+                            for bus in sel:
+                                buses.add(bus)
+
+                        elif isinstance(sel_obj, dev.Substation):
+                            root_bus = None
+                            sel = self.circuit.get_substation_buses(substation=sel_obj)
+                            for bus in sel:
+                                buses.add(bus)
+
                         else:
                             root_bus = None
 
                         if root_bus is not None:
                             buses.add(root_bus)
 
-        return buses
+        return buses, selected_objects
+
+    def get_selected_substations(self) -> Tuple[Set[dev.Substation], List[ALL_DEV_TYPES]]:
+        """
+        Get the substations matching the table selection
+        :return:  set of substations, list of selected objects originating the substation set
+        """
+        substations = set()
+        selected_objects: List[ALL_DEV_TYPES] = list()
+
+        model = self.ui.dataStructureTableView.model()
+
+        elm2se = dict()
+
+        # Associate country, community, region and municipality to substation
+        for se in self.circuit.substations:
+            for elm in [se.country, se.community, se.region, se.municipality]:
+                if elm is not None:
+                    if elm in elm2se:
+                        elm2se[elm].append(se)
+                    else:
+                        elm2se[elm] = [se]
+
+        # associate voltage levels to substations
+        for vl in self.circuit.voltage_levels:
+            if vl.substation is not None:
+                elm2se[vl] = [vl.substation]
+
+        # associate buses to substations
+        for bus in self.circuit.buses:
+            if bus.substation is not None:
+                elm2se[bus] = [bus.substation]
+
+        if model is not None:
+
+            sel_idx = self.ui.dataStructureTableView.selectedIndexes()
+            objects = model.objects
+
+            if len(objects) > 0:
+
+                if len(sel_idx) > 0:
+
+                    unique = {idx.row() for idx in sel_idx}
+
+                    for idx in unique:
+
+                        sel_obj: ALL_DEV_TYPES = model.objects[idx]
+                        selected_objects.append(sel_obj)
+
+                        se_list = elm2se.get(sel_obj, None)
+
+                        if se_list is not None:
+                            for se in se_list:
+                                substations.add(se)
+
+        return substations, selected_objects
 
     def delete_selected_objects(self):
         """
@@ -471,6 +539,8 @@ class ObjectsTableMain(DiagramsMain):
                 self.update_from_to_list_views()
                 self.update_date_dependent_combos()
 
+                self.show_info_toast(f"{len(selected_objects)} objects deleted")
+
     def duplicate_selected_objects(self):
         """
         Delete selection
@@ -514,8 +584,10 @@ class ObjectsTableMain(DiagramsMain):
                     self.view_objects_data()
                     self.update_from_to_list_views()
                     self.update_date_dependent_combos()
+
+                    self.show_info_toast(f"{len(selected_objects)} substations merged")
             else:
-                info_msg(f'Merge function not available for {selected_objects[0].device_type.value} devices')
+                self.show_warning_toast(f'Merge function not available for {selected_objects[0].device_type.value} devices')
 
     def copy_selected_idtag(self):
         """
@@ -531,6 +603,8 @@ class ObjectsTableMain(DiagramsMain):
             lst = list()
             cb.setText("\n".join([obj.idtag for obj in selected_objects]))
 
+            self.show_info_toast("Copied!")
+
     def add_objects_to_current_diagram(self):
         """
         Add selected DB objects to current diagram
@@ -567,30 +641,81 @@ class ObjectsTableMain(DiagramsMain):
         """
         Create a New diagram from a buses selection
         """
-        selected_objects = self.get_selected_table_buses()
+        selected_buses, selected_objects = self.get_selected_table_buses()
 
-        if len(selected_objects):
-            diagram = make_diagram_from_buses(circuit=self.circuit, buses=selected_objects)
+        if len(selected_buses):
+            diagram = make_diagram_from_buses(circuit=self.circuit,
+                                              buses=selected_buses,
+                                              name=selected_objects[0].name + " diagram")
 
             diagram_widget = SchematicWidget(gui=self,
                                              circuit=self.circuit,
                                              diagram=diagram,
                                              default_bus_voltage=self.ui.defaultBusVoltageSpinBox.value(),
-                                             time_index=self.get_diagram_slider_index(),
-                                             call_delete_db_element_func=self.call_delete_db_element)
+                                             time_index=self.get_diagram_slider_index())
 
             self.add_diagram_widget_and_diagram(diagram_widget=diagram_widget,
                                                 diagram=diagram)
             self.set_diagrams_list_view()
 
+            self.show_info_toast(f"{diagram.name} added")
+
+    def add_new_map_from_database_selection(self):
+        """
+        Create a New map from a buses selection
+        """
+        selected_substations, selected_objects = self.get_selected_substations()
+
+        if len(selected_substations):
+            cmap_text = self.ui.palette_comboBox.currentText()
+            cmap = self.cmap_dict[cmap_text]
+
+            expand_outside = yes_no_question(text="Expand outside of the given selection using the branches?",
+                                             title="Expand outside")
+
+            diagram = make_diagram_from_substations(
+                circuit=self.circuit,
+                substations=selected_substations,
+                use_flow_based_width=self.ui.branch_width_based_on_flow_checkBox.isChecked(),
+                min_branch_width=self.ui.min_branch_size_spinBox.value(),
+                max_branch_width=self.ui.max_branch_size_spinBox.value(),
+                min_bus_width=self.ui.min_node_size_spinBox.value(),
+                max_bus_width=self.ui.max_node_size_spinBox.value(),
+                arrow_size=self.ui.arrow_size_size_spinBox.value(),
+                palette=cmap,
+                default_bus_voltage=self.ui.defaultBusVoltageSpinBox.value(),
+                expand_outside=expand_outside,
+                name=f"{selected_objects[0].name} diagram"
+            )
+
+            default_tile_source = self.tile_name_dict[self.ui.tile_provider_comboBox.currentText()]
+            tile_source = self.tile_name_dict.get(diagram.tile_source, default_tile_source)
+
+            diagram_widget = GridMapWidget(
+                gui=self,
+                tile_src=tile_source,
+                start_level=diagram.start_level,
+                longitude=diagram.longitude,
+                latitude=diagram.latitude,
+                name=diagram.name,
+                circuit=self.circuit,
+                diagram=diagram,
+            )
+
+            self.add_diagram_widget_and_diagram(diagram_widget=diagram_widget,
+                                                diagram=diagram)
+            self.set_diagrams_list_view()
+
+            self.show_info_toast(f"{diagram.name} added")
+
     def crop_model_to_buses_selection(self):
         """
         Crop model to buses selection
         :return:
         """
-        selected_objects = self.get_selected_table_buses()
+        selected_buses, selected_objects = self.get_selected_table_buses()
 
-        if len(selected_objects):
+        if len(selected_buses):
 
             ok = yes_no_question(text="This will delete all buses and their connected elements that were not selected."
                                       "This cannot be undone and it is dangerous if you don't know"
@@ -600,12 +725,14 @@ class ObjectsTableMain(DiagramsMain):
             if ok:
                 to_be_deleted = list()
                 for bus in self.circuit.buses:
-                    if bus not in selected_objects:
+                    if bus not in selected_buses:
                         to_be_deleted.append(bus)
 
                 for bus in to_be_deleted:
                     self.circuit.delete_bus(obj=bus, delete_associated=True)
 
+                self.show_info_toast(f"{len(to_be_deleted)} buses removed from the model")
+
     def add_objects(self):
         """
         Add default objects objects
@@ -706,7 +833,7 @@ class ObjectsTableMain(DiagramsMain):
             elif elm_type == DeviceType.WireDevice.value:
 
                 name = f'Wire {len(self.circuit.wire_types) + 1}'
-                obj = dev.Wire(name=name, gmr=0.01, r=0.01, x=0)
+                obj = dev.Wire(name=name, r_ext=0.01, r=0.01, x=0)
                 self.circuit.add_wire(obj)
 
             elif elm_type == DeviceType.TransformerTypeDevice.value:
@@ -765,16 +892,14 @@ class ObjectsTableMain(DiagramsMain):
             if idx > -1:
                 if elm_type == DeviceType.OverheadLineTypeDevice.value:
 
-                    # pick the object
-                    tower = self.circuit.overhead_line_types[idx]
-
                     # launch editor
-                    self.tower_builder_window = TowerBuilderGUI(parent=self,
-                                                                tower=tower,
-                                                                wires_catalogue=self.circuit.wire_types)
+                    self.tower_builder_window = TowerBuilderGUI(
+                        tower=self.circuit.overhead_line_types[idx],
+                        wires_catalogue=self.circuit.wire_types
+                    )
+                    self.tower_builder_window.setModal(True)
                     self.tower_builder_window.resize(int(1.81 * 700.0), 700)
                     self.tower_builder_window.exec()
-                    self.collect_memory()
 
                 else:
 
@@ -979,13 +1104,14 @@ class ObjectsTableMain(DiagramsMain):
                 logger = bs.Logger()
 
                 t_idx = self.get_objects_time_index()
-
+                attr_list = list()
                 for index in indices:
                     i = index.row()
                     p_idx = index.column()
                     elm = model.objects[i]
                     attr = model.attributes[p_idx]
                     gc_prop = elm.registered_properties[attr]
+                    attr_list.append(attr)
                     if gc_prop.has_profile():
                         val = elm.get_value(prop=gc_prop, t_idx=t_idx)
                         profile = elm.get_profile_by_prop(prop=gc_prop)
@@ -996,6 +1122,9 @@ class ObjectsTableMain(DiagramsMain):
                 if logger.size():
                     logs_window = LogsDialogue("Assign to profile", logger=logger)
                     logs_window.exec()
+                else:
+                    lst = ", ".join(attr_list)
+                    self.show_info_toast(f"{lst} assigned to profile")
         else:
             info_msg("Select a cell or a column first", "Assign to profile")
 
@@ -1213,6 +1342,26 @@ class ObjectsTableMain(DiagramsMain):
                           icon_path=":/Icons/icons/copy.svg",
                           function_ptr=self.copy_selected_idtag)
 
+        gf.add_menu_entry(menu=context_menu,
+                          text="Crop model to buses selection",
+                          icon_path=":/Icons/icons/schematic.svg",
+                          function_ptr=self.crop_model_to_buses_selection)
+
+        gf.add_menu_entry(menu=context_menu,
+                          text="Copy table",
+                          icon_path=":/Icons/icons/copy.svg",
+                          function_ptr=self.copy_objects_data)
+
+        gf.add_menu_entry(menu=context_menu,
+                          text="Set value to column",
+                          icon_path=":/Icons/icons/copy2down.svg",
+                          function_ptr=self.set_value_to_column)
+
+        gf.add_menu_entry(menu=context_menu,
+                          text="Assign to profile",
+                          icon_path=":/Icons/icons/assign_to_profile.svg",
+                          function_ptr=self.assign_to_profile)
+
         context_menu.addSeparator()
 
         gf.add_menu_entry(menu=context_menu,
@@ -1240,25 +1389,12 @@ class ObjectsTableMain(DiagramsMain):
                           icon_path=":/Icons/icons/highlight2.svg",
                           function_ptr=self.highlight_based_on_property)
 
-        gf.add_menu_entry(menu=context_menu,
-                          text="Crop model to buses selection",
-                          icon_path=":/Icons/icons/schematic.svg",
-                          function_ptr=self.crop_model_to_buses_selection)
+        context_menu.addSeparator()
 
         gf.add_menu_entry(menu=context_menu,
-                          text="Copy table",
-                          icon_path=":/Icons/icons/copy.svg",
-                          function_ptr=self.copy_objects_data)
-
-        gf.add_menu_entry(menu=context_menu,
-                          text="Set value to column",
-                          icon_path=":/Icons/icons/copy2down.svg",
-                          function_ptr=self.set_value_to_column)
-
-        gf.add_menu_entry(menu=context_menu,
-                          text="Assign to profile",
-                          icon_path=":/Icons/icons/assign_to_profile.svg",
-                          function_ptr=self.assign_to_profile)
+                          text="New map from selection",
+                          icon_path=":/Icons/icons/map.svg",
+                          function_ptr=self.add_new_map_from_database_selection)
 
         # Convert global position to local position of the list widget
         mapped_pos = self.ui.dataStructureTableView.viewport().mapToGlobal(pos)
