@@ -20,6 +20,8 @@ Typical values of earth
 """
 
 
+
+
 class WiresTable(QtCore.QAbstractTableModel):
     """
     Wires table for the tower
@@ -29,13 +31,11 @@ class WiresTable(QtCore.QAbstractTableModel):
 
         QtCore.QAbstractTableModel.__init__(self, parent)
 
-        self.header = ['Name', 'R (Ohm/km)', 'GMR (m)', 'max current (kA)']
-
-        # self.index_prop = {0: 'name', 1: 'r', 2: 'GMR', 3: 'max_current'}
+        self.header = ['Name', 'R (Ohm/km)', 'Diameter (mm)', 'max current (kA)']
 
         self.converter = {0: str, 1: float, 2: float, 3: float}
 
-        self.editable = [True, True, True, True]
+        self.editable = [False, False, False, False]
 
         self.wires: List[Wire] = list()
 
@@ -66,7 +66,7 @@ class WiresTable(QtCore.QAbstractTableModel):
         checks if the name is used
         """
         n = len(self.wires)
-        for i in range(n-1, -1, -1):
+        for i in range(n - 1, -1, -1):
             if self.wires[i].name == name:
                 return True
         return False
@@ -100,7 +100,7 @@ class WiresTable(QtCore.QAbstractTableModel):
                 elif index.column() == 1:
                     return str(wire.R)
                 elif index.column() == 2:
-                    return str(wire.GMR)
+                    return str(wire.diameter)
                 elif index.column() == 3:
                     return str(wire.max_current)
 
@@ -196,7 +196,7 @@ class WiresCollection(QtCore.QAbstractTableModel):
         checks if the name is used
         """
         n = len(self.wires_in_tower)
-        for i in range(n-1, -1, -1):
+        for i in range(n - 1, -1, -1):
             if self.wires_in_tower[i].name == name:
                 return True
         return False
@@ -257,11 +257,10 @@ class WiresCollection(QtCore.QAbstractTableModel):
 
 class TowerModel(QtCore.QAbstractTableModel):
 
-    def __init__(self, parent=None, edit_callback=None, tower: Union[OverheadLineType, None] = None):
+    def __init__(self, edit_callback=None, tower: Union[OverheadLineType, None] = None):
         """
 
-        :param parent:
-        :param edit_callback:
+        :param edit_callback: compute function from the TowerBuilderGUI
         :param tower:
         """
 
@@ -275,6 +274,12 @@ class TowerModel(QtCore.QAbstractTableModel):
         # other properties
         self.edit_callback = edit_callback
 
+        # wire properties for edition (do not confuse with the properties of this very object...)
+        self.header = ['Wire', 'X (m)', 'Y (m)', 'Phase', "Circuit Index", "Phase name"]
+        self.index_prop = {0: 'name', 1: 'xpos', 2: 'ypos', 3: 'phase', 4: 'circuit_index', 5: 'phase_type'}
+        self.converter = {0: str, 1: float, 2: float, 3: int, 4: int, 5: str}
+        self.editable_wire = [False, True, True, True, False, False]
+
     def __str__(self):
         return self.tower.name
 
@@ -306,7 +311,7 @@ class TowerModel(QtCore.QAbstractTableModel):
         :param wire: Wire object
         """
         n = len(self.tower.wires_in_tower.data)
-        for i in range(n-1, -1, -1):
+        for i in range(n - 1, -1, -1):
             if self.tower.wires_in_tower.data[i].name == wire.name:
                 self.delete(i)
 
@@ -317,7 +322,7 @@ class TowerModel(QtCore.QAbstractTableModel):
         :return:
         """
         n = len(self.tower.wires_in_tower.data)
-        for i in range(n-1, -1, -1):
+        for i in range(n - 1, -1, -1):
             if self.tower.wires_in_tower.data[i].name == wire.name:
                 return True
 
@@ -327,7 +332,7 @@ class TowerModel(QtCore.QAbstractTableModel):
         :param index:
         :return:
         """
-        if self.tower.editable_wire[index.column()]:
+        if self.editable_wire[index.column()]:
             return QtCore.Qt.ItemFlag.ItemIsEditable | QtCore.Qt.ItemFlag.ItemIsEnabled | QtCore.Qt.ItemFlag.ItemIsSelectable
         else:
             return QtCore.Qt.ItemFlag.ItemIsEnabled
@@ -346,7 +351,7 @@ class TowerModel(QtCore.QAbstractTableModel):
         :param parent:
         :return:
         """
-        return len(self.tower.header)
+        return len(self.header)
 
     def data(self, index, role=QtCore.Qt.ItemDataRole.DisplayRole):
         """
@@ -357,7 +362,7 @@ class TowerModel(QtCore.QAbstractTableModel):
         """
         if index.isValid():
             if role == QtCore.Qt.ItemDataRole.DisplayRole:
-                val = getattr(self.tower.wires_in_tower.data[index.row()], self.tower.index_prop[index.column()])
+                val = getattr(self.tower.wires_in_tower.data[index.row()], self.index_prop[index.column()])
                 return str(val)
         return None
 
@@ -374,7 +379,7 @@ class TowerModel(QtCore.QAbstractTableModel):
         """
         if role == QtCore.Qt.ItemDataRole.DisplayRole:
             if orientation == QtCore.Qt.Orientation.Horizontal:
-                return self.tower.header[section]
+                return self.header[section]
 
     def setData(self, index, value, role=QtCore.Qt.ItemDataRole.DisplayRole):
         """
@@ -383,26 +388,20 @@ class TowerModel(QtCore.QAbstractTableModel):
         :param value:
         :param role:
         """
-        if self.tower.editable_wire[index.column()]:
+        if self.editable_wire[index.column()]:
             wire = self.tower.wires_in_tower.data[index.row()]
-            attr = self.tower.index_prop[index.column()]
+            attr = self.index_prop[index.column()]
 
             try:
-                val = self.tower.converter[index.column()](value)
+                val = self.converter[index.column()](value)
             except ValueError:
                 val = 0
             except TypeError:
                 val = 0
 
-            # correct the phase to the correct range
-            if attr == 'phase':
-                if val < 0 or val > 3:
-                    val = 0
-
             setattr(wire, attr, val)
 
             if self.edit_callback is not None:
                 self.edit_callback()
 
         return True
-
