@@ -941,6 +941,9 @@ class ArrayTableModel(QAbstractTableModel):
         for i in range(len(self._data)):
             self._data[i] = np.insert(self._data[i], position, [0] * rows)
 
+        # set active to true
+        self._data[0][-1] = 1
+
         self.endInsertRows()
         return True
 
@@ -1033,7 +1036,7 @@ if __name__ == "__main__":
     #                                 answer2="stay here")
     #
     # window.show()
-    # sys.exit(app.exec_())
+    # sys.exit(app.exec())
 
     app = QApplication(sys.argv)
     window = ArrayEditor()
